@@ -1,5 +1,5 @@
 __author__ = "Shyue Ping Ong"
-__date__ = "Jun 23 2013"
-__version__ = "0.4.3"
+__date__ = "Oct 12 2013"
+__version__ = "0.4.5"
 
 from custodian import Custodian
